(*************************************************************************

     $RCSfile: OLPrefs.mod $
  Description: Preferences editor for OL.

   Created by: fjc (Frank Copeland)
    $Revision: 1.4 $
      $Author: fjc $
        $Date: 1995/06/04 23:07:29 $

  Copyright  1995, Frank Copeland
  This module forms part of the OL program
  See OL.doc for conditions of use and distribution

  Log entries are at the end of the file.

*************************************************************************)

<* STANDARD- *>

MODULE OLPrefs;

IMPORT
  SYS := SYSTEM, Kernel, Errors, e := Exec, u := Utility, d := Dos,
  OLSettings, s := OLPrefsStrings, wb := Workbench, OLPrefsGUI;

CONST

  CopyrightStr = "Copyright  1995 Frank Copeland\n";

(* -- Command line template and parsing ------------------------------- *)

CONST

  template = "FROM";

  optFROM      = 0;
  optCount     = 1;

VAR

  rdArgs : d.RDArgsPtr;
  args : ARRAY optCount OF SYS.LONGWORD;

  (* These are filled in by ParseArgs() *)

  from : e.LSTRPTR;

(*
** Lock on starting directory when run from Workbench.
*)

VAR
  startDir : d.FileLockPtr;

(*------------------------------------*)
PROCEDURE* Cleanup (VAR rc : LONGINT);

  VAR oldDir : d.FileLockPtr;

BEGIN (* Cleanup *)
  IF rdArgs # NIL THEN
    d.FreeArgs (rdArgs);
    d.FreeDosObject (d.rdArgs, rdArgs);
    rdArgs := NIL
  END;
  s.CloseCatalog();
  IF Kernel.fromWorkbench THEN oldDir := d.CurrentDir (startDir) END
END Cleanup;

(*------------------------------------*)
PROCEDURE Init ();

BEGIN (* Init *)
  Kernel.SetCleanup (Cleanup);
  s.OpenCatalog (NIL, "");

  rdArgs := d.AllocDosObjectTags (d.rdArgs, u.end);
  IF (rdArgs = NIL) THEN
    OLPrefsGUI.Notice (s.msg1); HALT (d.warn)
  END
END Init;

(*------------------------------------*)
PROCEDURE ParseArgs ();

  VAR string : e.LSTRPTR; ignore : BOOLEAN;

BEGIN (* ParseArgs *)
  from := SYS.VAL (e.LSTRPTR, args [optFROM]);
  IF from = NIL THEN ignore := OLSettings.LoadPrefs ("OL.prefs")
  ELSE ignore := OLSettings.LoadPrefs (from^)
  END
END ParseArgs;

(*------------------------------------*)
PROCEDURE Main ();

  (*------------------------------------*)
  PROCEDURE WbArgs ();

    VAR
      wbStartup : wb.WBStartupPtr;
      wbArg     : wb.WBArg;

  BEGIN (* WbArgs *)
    wbStartup := SYS.VAL (wb.WBStartupPtr, Kernel.WBenchMsg);
    IF wbStartup.numArgs > 2 THEN OLPrefsGUI.Notice (s.msg2); HALT (d.warn) END;

    wbArg := wbStartup.argList [wbStartup.numArgs-1];
    startDir := d.CurrentDir (wbArg.lock);

    IF wbStartup.numArgs = 2 THEN
      args [optFROM] := wbArg.name
    END
  END WbArgs;

  (*------------------------------------*)
  PROCEDURE CliArgs ();
    VAR ignore : BOOLEAN;
  BEGIN (* CliArgs *)
    IF d.OldReadArgs (template, args, rdArgs) = NIL THEN
      ignore := d.PrintFault (d.IoErr(), "");
      HALT (d.warn)
    END
  END CliArgs;

BEGIN (* Main *)
  IF Kernel.fromWorkbench THEN WbArgs()
  ELSE CliArgs()
  END;
  ParseArgs();
  OLPrefsGUI.Open;
END Main;

BEGIN (* OLPrefs *)
  ASSERT (e.SysBase.libNode.version >= 37);
  Errors.Init;

  Init();
  Main()
END OLPrefs.

(*************************************************************************

  $Log: OLPrefs.mod $
# Revision 1.4  1995/06/04  23:07:29  fjc
# - Converted to a pure GUI program. Removed most of the
#   command-line processing.
#
*************************************************************************)
